#  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License").
#    You may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import json
import logging
import os
from typing import Any, Dict, List, Optional, Tuple, cast

import seedfarmer.errors
import seedfarmer.mgmt.module_info as mi
from seedfarmer import config
from seedfarmer.mgmt.module_info import get_module_metadata
from seedfarmer.models.manifests import DeploymentManifest, ModuleManifest, ModuleParameter
from seedfarmer.services.session_manager import SessionManager
from seedfarmer.types.parameter_types import EnvVar, EnvVarType
from seedfarmer.utils import upper_snake_case

_logger: logging.Logger = logging.getLogger(__name__)


def generate_export_env_params(metadata: Optional[Dict[str, Any]]) -> Optional[List[str]]:
    envs: List[str] = []
    if metadata is not None:
        envs = [
            f"export {config.PROJECT.upper()}_PARAMETER_{upper_snake_case(k)}='{metadata[k]}'" for k in metadata.keys()
        ]
    return envs


def generate_export_raw_env_params(metadata: Optional[Dict[str, Any]]) -> Optional[List[str]]:
    envs: List[str] = []
    if metadata is not None:
        envs = [f"export {upper_snake_case(k)}='{metadata[k]}'" for k in metadata.keys()]
    return envs


def load_parameter_values(
    deployment_name: str,
    parameters: List[ModuleParameter],
    deployment_manifest: DeploymentManifest,
    target_account: Optional[str],
    target_region: Optional[str],
) -> List[ModuleParameter]:
    parameter_values = []
    parameter_values_cache: Dict[Tuple[str, str, str], Any] = {}
    for parameter in parameters:
        if _logger.isEnabledFor(logging.DEBUG):
            _logger.debug("parameter: %s", parameter.model_dump())

        if parameter.value is not None:
            _logger.debug("static parameter value: %s", parameter.value)
            parameter_values.append(parameter)
        elif parameter.value_from:
            if parameter.value_from.module_metadata:
                module_metatdata = _module_metatdata(
                    deployment_name, parameter, parameter_values_cache, deployment_manifest
                )
                if module_metatdata:
                    parameter_values.append(module_metatdata)
                else:
                    raise seedfarmer.errors.InvalidManifestError(
                        f"The module metadata parameter ({parameter.value_from.module_metadata}) is not available"
                    )
            elif parameter.value_from.env_variable:
                if parameter.value_from.env_variable in os.environ:
                    parameter_values.append(
                        ModuleParameter(name=parameter.name, value=os.getenv(parameter.value_from.env_variable, ""))
                    )
                else:
                    raise seedfarmer.errors.InvalidManifestError(
                        f"The environment variable ({parameter.value_from.env_variable}) is not available"
                    )
            elif parameter.value_from.parameter_store:
                parameter_values.append(
                    ModuleParameter(
                        name=parameter.name,
                        value=EnvVar(value=parameter.value_from.parameter_store, type=EnvVarType.PARAMETER_STORE),
                    ),
                )
            elif parameter.value_from.secrets_manager:
                parameter_values.append(
                    ModuleParameter(
                        name=parameter.name,
                        value=EnvVar(value=parameter.value_from.secrets_manager, type=EnvVarType.SECRETS_MANAGER),
                    ),
                )
            elif parameter.value_from.parameter_value:
                p_value = deployment_manifest.get_parameter_value(
                    parameter=parameter.value_from.parameter_value, account_id=target_account, region=target_region
                )
                if p_value is not None:
                    p_value = str(p_value) if isinstance(p_value, str) else json.dumps(p_value)
                    parameter_values.append(ModuleParameter(name=parameter.name, value=p_value))
                else:
                    raise seedfarmer.errors.InvalidManifestError(
                        f"The parameter value defined ({parameter.value_from.parameter_value}) is not available"
                    )

    return parameter_values


def resolve_params_for_checksum(
    deployment_manifest: DeploymentManifest,
    module: ModuleManifest,
    group_name: str,
) -> None:
    for param in module.parameters:
        if param.value_from and param.value_from.parameter_store:
            if ":" in param.value_from.parameter_store:
                raise seedfarmer.errors.InvalidConfigurationError(
                    f"CodeBuild does not support Versioned SSM Parameters -- see {group_name}-{module.name}"
                )
            param.version = mi.get_ssm_parameter_version(
                ssm_parameter_name=param.value_from.parameter_store,
                session=SessionManager()
                .get_or_create()
                .get_deployment_session(
                    account_id=cast(str, module.get_target_account_id()),
                    region_name=cast(str, module.target_region),
                ),
            )

        elif param.value_from and param.value_from.secrets_manager:
            param_name = param.value_from.secrets_manager
            version_ref = None
            if ":" in param_name:
                parsed = param_name.split(":")
                param_name = parsed[0]
                version_ref = parsed[2] if len(parsed) == 3 else None

            param.version = mi.get_secrets_version(
                secret_name=param_name,
                version_ref=version_ref,
                session=SessionManager()
                .get_or_create()
                .get_deployment_session(
                    account_id=cast(str, module.get_target_account_id()),
                    region_name=cast(str, module.target_region),
                ),
            )
        elif param.value_from and param.value_from.parameter_value:
            p_value = deployment_manifest.get_parameter_value(
                parameter=param.value_from.parameter_value,
                account_alias=module.target_account,
                region=module.target_region,
            )
            if p_value is not None:
                param.resolved_value = str(p_value) if isinstance(p_value, str) else json.dumps(p_value)
            else:
                raise seedfarmer.errors.InvalidManifestError(
                    f"The parameter value defined ({param.value_from.parameter_value}) is not available"
                )

        elif param.value_from and param.value_from.env_variable:
            if param.value_from.env_variable in os.environ:
                param.resolved_value = os.getenv(param.value_from.env_variable)
            else:
                raise seedfarmer.errors.InvalidManifestError(
                    f"The environment variable ({param.value_from.env_variable}) is not available"
                )


def _get_param_value_cache(
    d_name: str,
    g_name: str,
    m_name: str,
    parameter_values_cache: Dict[Tuple[str, str, str], Any],
    deployment_manifest: DeploymentManifest,
) -> Dict[Any, Any]:
    if (d_name, g_name, m_name) not in parameter_values_cache:
        module = deployment_manifest.get_module(group=g_name, module=m_name)
        if module is not None:
            module_session = (
                SessionManager()
                .get_or_create()
                .get_deployment_session(
                    account_id=cast(str, module.get_target_account_id()), region_name=cast(str, module.target_region)
                )
            )
            parameter_values_cache[(d_name, g_name, m_name)] = get_module_metadata(
                d_name, g_name, m_name, session=module_session
            )
        else:
            return {}
    return cast(Dict[Any, Any], parameter_values_cache[(d_name, g_name, m_name)])


def _module_metatdata(
    deployment_name: str,
    parameter: ModuleParameter,
    parameter_values_cache: Dict[Tuple[str, str, str], Any],
    deployment_manifest: DeploymentManifest,
) -> Optional[ModuleParameter]:
    if parameter.value_from and parameter.value_from.module_metadata:
        group = parameter.value_from.module_metadata.group
        module_name = parameter.value_from.module_metadata.name
        _logger.debug("Loading metadata for dependency group, module: %s, %s" % (group, module_name))

        # Ensure we only retrieve the SSM Parameter value once per module
        parameter_value = _get_param_value_cache(
            deployment_name, group, module_name, parameter_values_cache, deployment_manifest
        )
        _logger.debug("loaded parameter value: %s", parameter_value)

        parameter_value = (
            parameter_value.get(parameter.value_from.module_metadata.key, None)  # type: ignore[assignment]
            if parameter_value is not None and parameter.value_from.module_metadata.key is not None
            else parameter_value
        )
        _logger.debug("parsed parameter value: %s", parameter_value)

        if parameter_value is not None:
            return ModuleParameter(
                name=parameter.name,
                value=parameter_value,
            )
    return None



#  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License").
#    You may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.


import logging
from typing import List, Optional

import click

from seedfarmer import DEBUG_LOGGING_FORMAT, config, enable_debug
from seedfarmer.commands import bootstrap_target_account, bootstrap_toolchain_account
from seedfarmer.output_utils import print_bolded

_logger: logging.Logger = logging.getLogger(__name__)


def _load_project() -> str:
    try:
        _logger.info("No --project provided, attempting load from seedfarmer.yaml")
        return config.PROJECT
    except FileNotFoundError:
        print_bolded("Unable to determine project to bootstrap, one of --project or a seedfarmer.yaml is required")
        raise click.ClickException("Failed to determine project identifier")


@click.group(name="bootstrap", help="Bootstrap (initialize) a Toolchain or Target account")
def bootstrap() -> None:
    """Bootstrap a Toolchain or Target account"""
    pass


@bootstrap.command(
    name="toolchain",
    help="Bootstrap a Toolchain account.",
)
@click.option(
    "--project",
    "-p",
    help="Project identifier",
    required=False,
    default=None,
)
@click.option(
    "--trusted-principal",
    "-t",
    help="""ARN of Principals trusted to assume the Toolchain Role.
    This can be used multiple times to create a list.""",
    multiple=True,
    required=True,
)
@click.option(
    "--permissions-boundary",
    "-b",
    help="ARN of a Managed Policy to set as the Permission Boundary on the Toolchain Role",
    required=False,
    default=None,
)
@click.option(
    "--as-target/--not-as-target",
    type=bool,
    default=False,
    help="Optionally also bootstrap the account as a Target account",
    required=False,
    show_default=True,
)
@click.option(
    "--synth/--no-synth",
    type=bool,
    default=False,
    help="Synthesize a CFN bootstrap template only...do not deploy",
    required=False,
    show_default=True,
)
@click.option(
    "--profile",
    default=None,
    help="The AWS profile to use to initiate a session",
    required=False,
)
@click.option(
    "--region",
    default=None,
    help="AWS region to use to initiate a session",
    required=False,
)
@click.option(
    "--qualifier",
    default=None,
    help="""A qualifier to append to toolchain role (alpha-numeric char max length of 6).
    If used, it MUST be used on every seedfarmer command.""",
    required=False,
)
@click.option(
    "--role-prefix",
    default=None,
    help="An IAM path prefix to use with the seedfarmer roles.",
    required=False,
)
@click.option(
    "--policy-prefix",
    default=None,
    help="An IAM path prefix to use with the seedfarmer policies.",
    required=False,
)
@click.option(
    "--policy-arn",
    "-pa",
    help="""ARN of existing Policy to attach to Target Role (Deploymenmt Role)
    This can be use multiple times, but EACH policy MUST be valid in the Target Account.
    The `--as-target` flag must be used if passing in policy arns as they are applied to
    the Deployment Role only.""",
    multiple=True,
    required=False,
    default=[],
)
@click.option("--debug/--no-debug", default=False, help="Enable detail logging", show_default=True)
def bootstrap_toolchain(
    project: Optional[str],
    trusted_principal: List[str],
    permissions_boundary: Optional[str],
    policy_arn: Optional[List[str]],
    profile: Optional[str],
    region: Optional[str],
    qualifier: Optional[str],
    role_prefix: Optional[str],
    policy_prefix: Optional[str],
    as_target: bool,
    synth: bool,
    debug: bool,
) -> None:
    if debug:
        enable_debug(format=DEBUG_LOGGING_FORMAT)
    if project is None:
        project = _load_project()
    _logger.debug("Bootstrapping a Toolchain account for Project %s", project)
    if len(policy_arn) > 0 and not as_target:  # type: ignore
        raise click.ClickException("Cannot set PolicyARNS when the `-as-target` flag is not set.")

    bootstrap_toolchain_account(
        project_name=project,
        principal_arns=trusted_principal,
        permissions_boundary_arn=permissions_boundary,
        policy_arns=policy_arn,
        profile=profile,
        qualifier=qualifier,
        role_prefix=role_prefix,
        policy_prefix=policy_prefix,
        region_name=region,
        synthesize=synth,
        as_target=as_target,
    )


@bootstrap.command(
    name="target",
    help="Bootstrap a Target account.",
)
@click.option(
    "--project",
    "-p",
    help="Project identifier",
    required=False,
    default=None,
)
@click.option(
    "--toolchain-account",
    "-t",
    required=True,
    help="Account Id of the Toolchain account trusted to assume the Target account's Deployment Role",
)
@click.option(
    "--permissions-boundary",
    "-b",
    help="ARN of a Managed Policy to set as the Permission Boundary on the Toolchain Role",
    required=False,
    default=None,
)
@click.option(
    "--synth/--no-synth",
    type=bool,
    default=False,
    help="Synthesize a CFN template only...do not deploy",
    required=False,
    show_default=True,
)
@click.option(
    "--profile",
    default=None,
    help="The AWS profile to use to initiate a session",
    required=False,
)
@click.option(
    "--region",
    default=None,
    help="AWS region to use to initiate a session",
    required=False,
)
@click.option(
    "--qualifier",
    default=None,
    help="""A qualifier to append to target role (alpha-numeric char max length of 6).
    If used on the toolchain account, it should be used here!""",
    required=False,
)
@click.option(
    "--role-prefix",
    default=None,
    help="An IAM path prefix to use with the seedfarmer roles.",
    required=False,
)
@click.option(
    "--policy-arn",
    "-pa",
    help="""ARN of existing Policy to attach to Target Role (Deploymenmt Role)
    This can be use multiple times to create a list, but EACH policy MUST be valid in the Target Account""",
    multiple=True,
    required=False,
    default=[],
)
@click.option("--debug/--no-debug", default=False, help="Enable detail logging", show_default=True)
def bootstrap_target(
    project: Optional[str],
    toolchain_account: str,
    permissions_boundary: Optional[str],
    policy_arn: Optional[List[str]],
    profile: Optional[str],
    region: Optional[str],
    qualifier: Optional[str],
    role_prefix: Optional[str],
    synth: bool,
    debug: bool,
) -> None:
    if debug:
        enable_debug(format=DEBUG_LOGGING_FORMAT)
    if project is None:
        project = _load_project()
    _logger.debug("Bootstrapping a Target account for Project %s", project)
    bootstrap_target_account(
        toolchain_account_id=toolchain_account,
        project_name=project,
        profile=profile,
        region_name=region,
        qualifier=qualifier,
        role_prefix=role_prefix,
        permissions_boundary_arn=permissions_boundary,
        policy_arns=policy_arn,
        synthesize=synth,
    )



#  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License").
#    You may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from typing import Optional, Tuple

import click

import seedfarmer.commands._seedkit_commands as sk_commands
from seedfarmer.services.session_manager import SessionManager, SessionManagerLocalImpl

# _logger: logging.Logger = logging.getLogger(__name__)


@click.group(name="seedkit", help="Top Level command to support seedkits in SeedFarmer")
def seedkit() -> None:
    "Manage the seedkit"
    pass


@seedkit.command(
    name="deploy",
    help="""Deploy a seedkit in the specified account and region.  There can
            be only one per seedfarmer project and is region-based.
            """,
)
@click.argument(
    "project_name",
    type=str,
    required=True,
)
@click.option("--policy-arn", required=False, type=str, multiple=True, default=[])
@click.option(
    "--deploy-codeartifact/--skip-codeartifact",
    default=False,
    help="Deploy the optional CodeArtifact Domain and Repository",
    show_default=True,
)
@click.option(
    "--profile",
    default=None,
    help="AWS Credentials profile to use for boto3 commands",
    show_default=True,
)
@click.option(
    "--region",
    default=None,
    help="AWS region to use for boto3 commands",
    show_default=True,
)
@click.option(
    "--vpc-id",
    help="The VPC ID that the Codebuild Project resides in (only 1)",
    required=False,
    default=None,
)
@click.option(
    "--subnet-id",
    help="A subnet that the Codebuild Project resides in (many can be passed in)",
    multiple=True,
    required=False,
    default=[],
)
@click.option(
    "--sg-id",
    help="A Securtiy Group in the VPC that the Codebuild Project can leverage (up to 5)",
    multiple=True,
    required=False,
    default=[],
)
@click.option(
    "--permissions-boundary-arn",
    "-b",
    help="ARN of a Managed Policy to set as the Permission Boundary on the CodeBuild Role",
    required=False,
    default=None,
)
@click.option(
    "--synth/--no-synth",
    type=bool,
    default=False,
    help="Synthesize seedkit template only. Do not deploy",
    required=False,
    show_default=True,
)
@click.option(
    "--debug/--no-debug",
    default=False,
    help="Enable detailed logging.",
    show_default=True,
)
def deploy_seedkit(
    project_name: str,
    policy_arn: Tuple[str, ...],
    deploy_codeartifact: bool,
    profile: Optional[str],
    region: Optional[str],
    debug: bool,
    vpc_id: Optional[str],
    subnet_id: Tuple[str, ...],
    sg_id: Tuple[str, ...],
    permissions_boundary_arn: Optional[str],
    synth: bool,
) -> None:
    SessionManager.bind(SessionManagerLocalImpl())  # should ALWAYS use local profile info
    session = SessionManager().get_or_create(region_name=region, profile=profile).toolchain_session
    sk_commands.deploy_seedkit(
        seedkit_name=project_name,
        managed_policy_arns=[p for p in policy_arn],
        deploy_codeartifact=deploy_codeartifact,
        session=session,
        vpc_id=vpc_id,
        subnet_ids=[s for s in subnet_id],
        security_group_ids=[sg for sg in sg_id],
        permissions_boundary_arn=permissions_boundary_arn,
        synthesize=synth,
    )


@seedkit.command(name="destroy")
@click.argument(
    "project_name",
    type=str,
    required=True,
)
@click.option(
    "--profile",
    default=None,
    help="AWS Credentials profile to use for boto3 commands",
    show_default=True,
)
@click.option(
    "--region",
    default=None,
    help="AWS region to use for boto3 commands",
    show_default=True,
)
@click.option(
    "--debug/--no-debug",
    default=False,
    help="Enable detailed logging.",
    show_default=True,
)
def destroy_seedkit(project_name: str, profile: Optional[str], region: Optional[str], debug: bool) -> None:
    SessionManager.bind(SessionManagerLocalImpl())
    session = SessionManager().get_or_create(region_name=region, profile=profile).toolchain_session
    sk_commands.destroy_seedkit(seedkit_name=project_name, session=session)



#  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License").
#    You may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import click

import seedfarmer.mgmt.module_init as minit


@click.group(name="init", help="Initialize a project or module")
def init() -> None:
    """Initialize a project or module"""
    pass


@init.command(name="project", help="Initialize a project.")
@click.option(
    "--project-name",
    "-p",
    default=None,
    help="The name of the project. If None, seedfarmer.yaml must be at the dir where cli invoked.",
    required=False,
)
@click.option(
    "--project_dir",
    "-pd",
    default=None,
    help="The name of the directory that houses the project if not to be the same as the project name",
    required=False,
)
@click.option(
    "--template-url",
    "-t",
    default="https://github.com/awslabs/seed-farmer.git",
    help="The template URL. If not specified, the default template repo is `https://github.com/awslabs/seed-farmer`",
    required=False,
)
@click.option(
    "--template-branch",
    "-b",
    default="init-project",
    help="The Branch on the template repository. If not specified, the default template branch is `init-project`",
    required=False,
)
def init_project(template_url: str, template_branch: str, project_name: str, project_dir: str) -> None:
    minit.create_project(
        template_url=template_url, template_branch=template_branch, project_name=project_name, project_dir=project_dir
    )


@init.command(name="module", help="Initialize a new module")
@click.option(
    "--group-name",
    "-g",
    type=str,
    help="The group the module belongs to. The `group` is created if it doesn't exist",
    required=False,
    default=None,
)
@click.option("--module-name", "-m", type=str, help="The module name", required=True)
@click.option(
    "--module-type",
    "-mt",
    type=str,
    help="The type of module code deployed...only 'cdkv2' is accepted if used here",
    required=False,
    default=None,
)
@click.option(
    "--template-url",
    "-t",
    default="https://github.com/awslabs/seed-farmer.git",
    help=("The template URL. If not specified, the default template repo is `https://github.com/awslabs/seed-farmer`"),
    required=False,
)
@click.option(
    "--template-branch",
    "-b",
    default="init-module",
    help="The Branch on the template repository. If not specified, the default template branch is `main`",
    required=False,
)
@click.option("--debug/--no-debug", default=False, help="Enable detail logging", show_default=True)
def init_module(
    group_name: str, module_name: str, module_type: str, template_url: str, template_branch: str, debug: bool
) -> None:
    minit.create_module_dir(
        group_name=group_name,
        module_name=module_name,
        module_type=module_type,
        template_url=template_url,
        template_branch=template_branch,
    )



#!/usr/bin/env python

import argparse
import json
import logging
import os
import subprocess
from typing import Dict, cast

import boto3
from botocore.exceptions import ClientError

logger = logging.getLogger()
logger.setLevel(logging.INFO)


def get_secret(secret_name: str) -> Dict[str, Dict[str, str]]:
    region_name = os.environ.get("AWS_DEFAULT_REGION")
    session = boto3.session.Session()
    client = session.client(service_name="secretsmanager", region_name=region_name)

    try:
        get_secret_value_response = client.get_secret_value(SecretId=secret_name)
    except ClientError:
        logger.info("Secret with SecretId '%s' could not be retrieved from SecretsManager", secret_name)
        return {}
    else:
        return cast(Dict[str, Dict[str, str]], json.loads(get_secret_value_response.get("SecretString", "{}")))


def main(url: str) -> None:
    secret_name = os.environ.get("SEEDFARMER_NPM_MIRROR_SECRET", "NO_SECRET")
    # Backwards Compatibility
    if secret_name == "NO_SECRET":
        secret_name = os.environ.get("AWS_CODESEEDER_NPM_MIRROR_SECRET", "NO_SECRET")
    elif secret_name == "NO_SECRET":
        secret_name = os.environ.get("AWS_CODESEEDER_MIRROR_SECRET", "NO_SECRET")

    if secret_name != "NO_SECRET":
        secret_name_key = secret_name.split("::")[0] if "::" in secret_name else secret_name
        key = secret_name.split("::")[1] if "::" in secret_name else "npm"
        creds = get_secret(secret_name=secret_name_key)
        if key in creds.keys():
            ssl_token = creds[key]["ssl_token"] if creds[key].get("ssl_token") else None
            print("Secret configured for npm auth")
            config_command = f"{url.replace('https:', '')}:_auth={ssl_token}"
            process = subprocess.Popen(
                ["npm", "config", "set", config_command.split("=")[0], config_command.split("=")[1]],
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
            )
            process.communicate()
    else:
        print("'NPM_MIRROR_SECRET' is not set")
    print(f"Calling npm config with {url}")
    subprocess.call(["npm", "config", "set", "registry", url])


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="URL to evaluate")
    parser.add_argument("url", type=str, help="The url to set in npm'")
    args = parser.parse_args()
    main(args.url)



#  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License").
#    You may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import logging
from collections import ChainMap
from typing import Any, Dict, List, Optional

from boto3 import Session

import seedfarmer.errors
from seedfarmer.models.manifests._deployment_manifest import DeploymentManifest
from seedfarmer.models.manifests._module_manifest import ModuleManifest
from seedfarmer.services import _codebuild as codebuild

_logger: logging.Logger = logging.getLogger(__name__)


def get_build_env_params(build_ids: List[str], session: Optional[Session] = None) -> Optional[Dict[str, Any]]:
    response = codebuild.get_build_data(build_ids=build_ids, session=session)
    env_params = (
        response["builds"][0]["environment"]["environmentVariables"]
        if (
            response
            and response.get("builds")
            and response["builds"][0].get("environment")
            and response["builds"][0]["environment"].get("environmentVariables")
        )
        else None
    )
    list_of_envs = [{entry_dict["name"]: entry_dict["value"]} for entry_dict in env_params] if env_params else None
    return dict(ChainMap(*list_of_envs)) if list_of_envs else None


def get_manifest_schema(type: str) -> Dict[str, Any]:
    if "deployment" == type.lower():
        return DeploymentManifest.model_json_schema()
    elif "module" == type.lower():
        return ModuleManifest.model_json_schema()
    else:
        raise seedfarmer.errors.SeedFarmerException("Schema type selection not in [deployment, module]")
